@@ -15,8 +15,8 @@
 #' @aliases forward.search
 #' @param data matrix or data.frame 
 #' @param model if a single numeric number declares number of factors to extract in 
-#' exploratory factor analysis. If \code{class(model)} is a sem (or OpenMx model if installed 
-#' from github) then a confirmatory approach is performed instead
+#' exploratory factor analysis. If \code{class(model)} is a sem (semmod), or lavaan (character), 
+#' then a confirmatory approach is performed instead
 #' @param criteria character strings indicating the forward search method
 #' Can contain \code{'LD'} for likelihood distance, \code{'mah'} for Mahalanobis
 #' distance, or \code{'res'} for model implied residuals 
@@ -64,32 +64,22 @@
 #' (FS.outlier <- forward.search(holzinger.outlier, model))
 #' plot(FS)
 #' plot(FS.outlier)
-#'
-#' #Confirmatory using OpenMx (requires github version, see ?faoutlier)
-#' manifests <- colnames(holzinger)
-#' latents <- c("F1","F2","F3")
-#' #specify model, mxData not necessary but useful to check if mxRun works
-#' model <- mxModel("Three Factor",
-#'       type="RAM",
-#'       manifestVars = manifests,
-#'       latentVars = latents,
-#'       mxPath(from="F1", to=manifests[1:3]),
-#' 	     mxPath(from="F2", to=manifests[4:6]),
-#' 	     mxPath(from="F3", to=manifests[7:9]),
-#'       mxPath(from=manifests, arrows=2),
-#'       mxPath(from=latents, arrows=2,
-#'             free=FALSE, values=1.0),
-#'       mxData(cov(holzinger), type="cov", numObs=nrow(holzinger))
-#' 	  )			
 #' 
-#' (FS <- forward.search(holzinger, model))	  
+#' #Confirmatory with lavaan 
+#' model <- 'F1 =~  Remndrs + SntComp + WrdMean
+#' F2 =~ MissNum + MxdArit + OddWrds
+#' F3 =~ Boots + Gloves + Hatchts'
+#' 
+#' (FS <- forward.search(holzinger, model))      
 #' (FS.outlier <- forward.search(holzinger.outlier, model))
 #' plot(FS)
 #' plot(FS.outlier)
+#'
+#'
 #' }
 forward.search <- function(data, model, criteria = c('LD', 'mah'), 
-	n.subsets = 1000, p.base= .4, na.rm = TRUE, digits = 5, print.messages = TRUE)
-{		
+	n.subsets = 1000, p.base= .4, na.rm = TRUE, digits = 5, print.messages = TRUE, ...)
+{	    
 	if(na.rm) data <- na.omit(data)
 	N <- nrow(data)
 	p <- ncol(data)
@@ -174,7 +164,7 @@ forward.search <- function(data, model, criteria = c('LD', 'mah'),
 	    STATISTICS <- rep(NA, n.subsets)
 	    sampleCov <- cov(data)	    
 	    for(i in 1:n.subsets){	        
-	        samplesemMod <- sem(model, cov(data[-i,]), N-1)
+	        samplesemMod <- sem(model, cov(data[-i,]), N-1, ...)
 	        STATISTICS[i] <- samplesemMod$criterion * (samplesemMod$N - 1)
 	    }        
         orgbaseID <- baseID <- Samples[ ,(min(STATISTICS) == STATISTICS)]
@@ -184,13 +174,13 @@ forward.search <- function(data, model, criteria = c('LD', 'mah'),
 	    orderentered <- c()
 	    for (LOOP in 1:length(nbaseID)){	
 	        tmpcov <- cov(basedata)
-	        basemodels[[LOOP]] <- sem(model, tmpcov, nrow(basedata))	    	
+	        basemodels[[LOOP]] <- sem(model, tmpcov, nrow(basedata), ...)	    	
 	        stat <- c()
 	        RANK <- rep(0, length(nbaseID))		
 			if(any(criteria == 'LD')){	
 				for(j in 1:length(nbaseID)){
 					tmpcov <- cov(rbind(basedata, data[nbaseID[j], ]))					
-					tmpmod <- sem(model, tmpcov, nrow(basedata) + 1)
+					tmpmod <- sem(model, tmpcov, nrow(basedata) + 1, ...)
 					stat[j] <- tmpmod$criterion * (tmpmod$N - 1)
 				}		
 				RANK <- RANK + rank(stat)
@@ -222,7 +212,7 @@ forward.search <- function(data, model, criteria = c('LD', 'mah'),
 	    	}
 	    }	
 	    tmpcov <- cov(data)	    
-	    basemodels[[LOOP+1]] <- sem(model, tmpcov, N)
+	    basemodels[[LOOP+1]] <- sem(model, tmpcov, N, ...)
 	    LDstat <- RMR <- Cooksstat <- c()		
 	    for(i in 1:(length(basemodels)-1)){
 	    	LDstat[i] <- basemodels[[i]]$criterion * (basemodels[[i]]$N - 1)			
@@ -243,90 +233,77 @@ forward.search <- function(data, model, criteria = c('LD', 'mah'),
 	    RMR[i+1] <- sqrt(2*sum(((C - Chat)^2) /	(ncol(C)*(ncol(C) + 1))))	
 	    ret <- list(LD=LDstat, RMR=RMR, gCD=Cooksstat, ord=orderentered)	    
 	}
-	##OPENMX## if(class(model) == "MxRAMModel" || class(model) == "MxModel" ){	
-	##OPENMX## 	STATISTICS <- rep(NA, n.subsets)
-	##OPENMX## 	mxMod <- model
-	##OPENMX## 	sampleMxData <- mxData(cov(data), type="cov", numObs = nrow(data))
-	##OPENMX## 	sampleMxMod <- mxRun(mxModel(mxMod, sampleMxData), silent = TRUE)
-	##OPENMX## 	for(i in 1:n.subsets){
-	##OPENMX## 		sampleMxData <- mxData(cov(data[Samples[ ,i], ]), type="cov", numObs = nrow(Samples))
-	##OPENMX## 		sampleMxMod <- mxRun(mxModel(mxMod, sampleMxData), silent = TRUE)
-	##OPENMX## 		STATISTICS[i] <- sampleMxMod@output$Minus2LogLikelihood - 
-	##OPENMX## 			sampleMxMod@output$SaturatedLikelihood 			
-	##OPENMX## 	}
-	##OPENMX## 	orgbaseID <- baseID <- Samples[ ,(min(STATISTICS) == STATISTICS)]
-	##OPENMX## 	nbaseID <- setdiff(ID, baseID)
-	##OPENMX## 	basedata <- data[baseID, ]
-	##OPENMX## 	basemodels <- list()
-	##OPENMX## 	orderentered <- c()
-	##OPENMX## 	for (LOOP in 1:length(nbaseID)){	
-	##OPENMX## 		tmpcov <- cov(basedata)
-	##OPENMX## 		Data <- mxData(tmpcov, type="cov", numObs = nrow(Samples))
-	##OPENMX## 		basemodels[[LOOP]] <- mxRun(mxModel(mxMod, Data), silent = TRUE)
-	##OPENMX## 		stat <- c()
-	##OPENMX## 		RANK <- rep(0, length(nbaseID))		
-	##OPENMX## 		if(any(criteria == 'LD')){	
-	##OPENMX## 			for(j in 1:length(nbaseID)){
-	##OPENMX## 				tmpcov <- cov(rbind(basedata, data[nbaseID[j], ]))
-	##OPENMX## 				sampleMxData <- mxData(tmpcov, type="cov", numObs = nrow(basedata) + 1)
-	##OPENMX## 				sampleMxMod <- mxRun(mxModel(mxMod, sampleMxData), silent = TRUE)
-	##OPENMX## 				stat[j] <- sampleMxMod@output$Minus2LogLikelihood - 
-	##OPENMX## 					sampleMxMod@output$SaturatedLikelihood	
-	##OPENMX## 			}		
-	##OPENMX## 			RANK <- RANK + rank(stat)
-	##OPENMX## 		}
-	##OPENMX## 		if(any(criteria == 'mah')){	
-	##OPENMX## 			stat <- mahalanobis(data[nbaseID, ], colMeans(data[baseID, ]), 
-	##OPENMX## 				cov(data[baseID, ]))
-	##OPENMX## 			RANK <- RANK + rank(stat)	
-	##OPENMX## 		}
-	##OPENMX## 		if(any(criteria == 'res')){
-	##OPENMX## 			stat <- c()
-	##OPENMX## 			for(j in 1:length(nbaseID)){					
-	##OPENMX## 				tmp <- obs.resid(rbind(basedata, data[nbaseID[j], ]), model)
-	##OPENMX## 				stat[j] <- tmp$std_res[nrow(basedata)+1, ] %*% 
-	##OPENMX## 					tmp$std_res[nrow(basedata)+1, ]
-	##OPENMX## 			}
-	##OPENMX## 			RANK <- RANK + rank(stat)
-	##OPENMX## 		}
-	##OPENMX## 		RANK <- rank(RANK)
-	##OPENMX## 		newID <- nbaseID[min(RANK) == RANK]
-	##OPENMX## 		if(length(newID) > 1) newID <- newID[1]
-	##OPENMX## 		orderentered <- c(orderentered, newID)
-	##OPENMX## 		baseID <- c(baseID, newID)
-	##OPENMX## 		nbaseID <- setdiff(ID, baseID)
-	##OPENMX## 		basedata <- data[baseID, ]
-	##OPENMX## 		if(print.messages){
-	##OPENMX## 			cat('Remaining iterations:', length(nbaseID), '\n')	
-	##OPENMX## 			flush.console()		
-	##OPENMX## 		}
-	##OPENMX## 	}	
-	##OPENMX## 	tmpcov <- cov(data)
-	##OPENMX## 	Data <- mxData(tmpcov, type="cov", numObs = nrow(data))
-	##OPENMX## 	basemodels[[LOOP+1]] <- mxRun(mxModel(mxMod, Data), silent = TRUE)		
-	##OPENMX## 	LDstat <- RMR <- Cooksstat <- c()		
-	##OPENMX## 	for(i in 1:(length(basemodels)-1)){
-	##OPENMX## 		LDstat[i] <- basemodels[[i]]@output$Minus2LogLikelihood - 
-	##OPENMX## 			basemodels[[i]]@output$SaturatedLikelihood			
-	##OPENMX## 		theta <- basemodels[[i]]@output$estimate	
-	##OPENMX## 		hess <- basemodels[[i]]@output$estimatedHessian		
-	##OPENMX## 		theta2 <- basemodels[[i+1]]@output$estimate
-	##OPENMX## 		Cooksstat[i] <- (theta-theta2) %*% solve(hess) %*% (theta-theta2)			
-	##OPENMX## 		Chat <- basemodels[[i]]@objective@info$expCov			
-	##OPENMX## 		C <- basemodels[[i]]@data@observed			
-	##OPENMX## 		RMR[i] <- sqrt(2*sum(((C - Chat)^2) /
-	##OPENMX## 			(ncol(C)*(ncol(C) + 1))))
-	##OPENMX## 	}		
-	##OPENMX## 	Cooksstat <- c(NA, Cooksstat)
-	##OPENMX## 	orderentered <- c(NA, orderentered)
-	##OPENMX## 	LDstat[i+1] <- basemodels[[i+1]]@output$Minus2LogLikelihood - 
-	##OPENMX## 			basemodels[[i+1]]@output$SaturatedLikelihood
-	##OPENMX## 	Chat <- basemodels[[i+1]]@objective@info$expCov	
-	##OPENMX## 	C <- basemodels[[i+1]]@data@observed			
-	##OPENMX## 	RMR[i+1] <- sqrt(2*sum(((C - Chat)^2) /
-	##OPENMX## 			(ncol(C)*(ncol(C) + 1))))	
-	##OPENMX## 	ret <- list(LD=LDstat, RMR=RMR, gCD=Cooksstat, ord=orderentered)
-	##OPENMX## }
+	if(class(model) == "character"){         
+	    if(!require(lavaan)) require(lavaan)        
+	    STATISTICS <- rep(NA, n.subsets)
+	    for(i in 1:n.subsets){	        
+	        samplesemMod <- lavaan::sem(model, data=data[-i,], ...)
+	        STATISTICS[i] <- samplesemMod@Fit@test[[1]]$stat
+	    }        
+	    orgbaseID <- baseID <- Samples[ ,(min(STATISTICS) == STATISTICS)]
+	    nbaseID <- setdiff(ID, baseID)
+	    basedata <- data[baseID, ]
+	    basemodels <- list()
+	    orderentered <- c()
+	    for (LOOP in 1:length(nbaseID)){	
+	        basemodels[[LOOP]] <- lavaan::sem(model, data=basedata, ...)	    	
+	        stat <- c()
+	        RANK <- rep(0, length(nbaseID))		
+	        if(any(criteria == 'LD')){	
+	            for(j in 1:length(nbaseID)){
+	                tmpmod <- lavaan::sem(model, data=data, ...)
+	                stat[j] <- tmpmod@Fit@test[[1]]$stat
+	            }		
+	            RANK <- RANK + rank(stat)
+	        }
+	        if(any(criteria == 'mah')){	
+	            stat <- mahalanobis(data[nbaseID, ], colMeans(data[baseID, ]), 
+	                                cov(data[baseID, ]))
+	            RANK <- RANK + rank(stat)	
+	        }
+	        if(any(criteria == 'res')){ 
+	            stat <- c()
+	            for(j in 1:length(nbaseID)){					
+	                tmp <- obs.resid(rbind(basedata, data[nbaseID[j], ]), model)
+	                stat[j] <- tmp$std_res[nrow(basedata)+1, ] %*% 
+	                    tmp$std_res[nrow(basedata)+1, ]
+	            }
+	            RANK <- RANK + rank(stat)
+	        }
+	        RANK <- rank(RANK)
+	        newID <- nbaseID[min(RANK) == RANK]
+	        if(length(newID) > 1) newID <- newID[1]
+	        orderentered <- c(orderentered, newID)
+	        baseID <- c(baseID, newID)
+	        nbaseID <- setdiff(ID, baseID)
+	        basedata <- data[baseID, ]
+	        if(print.messages){
+	            cat('Remaining iterations:', length(nbaseID), '\n')	
+	            flush.console()		
+	        }
+	    }		    
+	    basemodels[[LOOP+1]] <- lavaan::sem(model, data=data,  ...)
+	    LDstat <- RMR <- Cooksstat <- c()		
+	    for(i in 1:(length(basemodels)-1)){
+	        LDstat[i] <- basemodels[[i]]@Fit@test[[1]]$stat
+	        theta <- basemodels[[i]]@Fit@x
+	        vcov <- vcov(basemodels[[i]])
+	        theta2 <- basemodels[[i+1]]@Fit@x
+	        Cooksstat[i] <- (theta-theta2) %*% vcov %*% (theta-theta2)			
+	        Chat <- basemodels[[i]]@Fit@Sigma.hat[[1]]
+	        C <- basemodels[[i]]@SampleStats@cov[[1]]			
+	        RMR[i] <- sqrt(2*sum(((C - Chat)^2) /
+	                                 (ncol(C)*(ncol(C) + 1))))
+	    }		
+	    Cooksstat <- c(NA, Cooksstat)
+	    orderentered <- c(NA, orderentered)
+	    LDstat[i+1] <- basemodels[[i+1]]@Fit@test[[1]]$stat
+	    Chat <- basemodels[[i+1]]@Fit@Sigma.hat[[1]]
+	    C <- basemodels[[i+1]]@SampleStats@cov[[1]]
+	    RMR[i+1] <- sqrt(2*sum(((C - Chat)^2) /	(ncol(C)*(ncol(C) + 1))))	
+	    ret <- list(LD=LDstat, RMR=RMR, gCD=Cooksstat, ord=orderentered)	    
+	}
+	
 	class(ret) <- 'forward.search'
 	ret
 }
