@@ -12,8 +12,8 @@
 #' @aliases gCD
 #' @param data matrix or data.frame 
 #' @param model if a single numeric number declares number of factors to extract in 
-#' exploratory factor analysis. If \code{class(model)} is a sem (or OpenMx model if installed 
-#' from github) then a confirmatory approach is performed instead
+#' exploratory factor analysis. If \code{class(model)} is a sem (semmod), or lavaan (character), 
+#' then a confirmatory approach is performed instead
 #' @param na.rm logical; remove rows with missing data? Note that this is required for 
 #' EFA analysis
 #' @param digits number of digits to round in the final result
@@ -38,17 +38,18 @@
 #' plot(gCDresult)
 #' plot(gCDresult.outlier)
 #'
+#' #-------------------------------------------------------------------
 #' #Confirmatory with sem
 #' model <- specifyModel()
-#'	  F1 -> V1,    lam11
-#' 	  F1 -> V2,    lam21
-#' 	  F1 -> V3,    lam31
-#' 	  F2 -> V4,    lam41
-#' 	  F2 -> V5,    lam52
-#' 	  F2 -> V6,    lam62
-#' 	  F3 -> V7,    lam73
-#'	  F3 -> V8,    lam83
-#' 	  F3 -> V9,    lam93
+#'    F1 -> Remndrs,    lam11
+#' 	  F1 -> SntComp,    lam21
+#' 	  F1 -> WrdMean,    lam31
+#' 	  F2 -> MissNum,    lam41
+#' 	  F2 -> MxdArit,    lam52
+#' 	  F2 -> OddWrds,    lam62
+#' 	  F3 -> Boots,      lam73
+#'	  F3 -> Gloves,     lam83
+#' 	  F3 -> Hatchts,    lam93
 #' 	  F1 <-> F1,   NA,     1
 #' 	  F2 <-> F2,   NA,     1
 #' 	  F3 <-> F3,   NA,     1
@@ -58,29 +59,19 @@
 #' plot(gCDresult2)
 #' plot(gCDresult2.outlier)
 #'
-#' #Confirmatory using OpenMx (requires github version, see ?faoutlier)
-#' manifests <- colnames(holzinger)
-#' latents <- c("F1","F2","F3")
-#' #specify model, mxData not necessary but useful to check if mxRun works
-#' model <- mxModel("Three Factor",
-#'       type="RAM",
-#'       manifestVars = manifests,
-#'       latentVars = latents,
-#'       mxPath(from="F1", to=manifests[1:3]),
-#' 	     mxPath(from="F2", to=manifests[4:6]),
-#' 	     mxPath(from="F3", to=manifests[7:9]),
-#'       mxPath(from=manifests, arrows=2),
-#'       mxPath(from=latents, arrows=2,
-#'             free=FALSE, values=1.0),
-#'       mxData(cov(holzinger), type="cov", numObs=nrow(holzinger))
-#' 	  )			
-#' 	  
-#' (gCDresult2 <- gCD(holzinger, model))	  
-#' (gCDresult2.outlier <- gCD(holzinger.outlier, model))
+#' #-------------------------------------------------------------------
+#' #Confirmatory with lavaan
+#' model <- 'F1 =~  Remndrs + SntComp + WrdMean
+#' F2 =~ MissNum + MxdArit + OddWrds
+#' F3 =~ Boots + Gloves + Hatchts'
+#' 
+#' (gCDresult2 <- gCD(holzinger, model, orthogonal=TRUE))      
+#' (gCDresult2.outlier <- gCD(holzinger.outlier, model, orthogonal=TRUE))
 #' plot(gCDresult2)
 #' plot(gCDresult2.outlier)
+#' 
 #' }
-gCD <- function(data, model, na.rm = TRUE, digits = 5)
+gCD <- function(data, model, na.rm = TRUE, digits = 5, ...)
 {	
 	if(na.rm) data <- na.omit(data)
 	N <- nrow(data)
@@ -102,13 +93,13 @@ gCD <- function(data, model, na.rm = TRUE, digits = 5)
 		ret <- list(dfbetas = DFBETAS, gCD = gCD)
 	}	
 	if(class(model) == "semmod"){
-	    mod <- sem(model, data=data)
+	    mod <- sem::sem(model, data=data, ...)
 	    theta <- mod$coeff		
 	    gCD <- c()	
 	    DFBETAS <- matrix(0, N, length(theta))
 	    for(i in 1:nrow(data)){
 	        tmp1 <- cov(data[-i, ])
-	        tmp2 <- sem(model, tmp1, N-1)
+	        tmp2 <- sem::sem(model, tmp1, N-1, ...)
 	        vcovmat <- tmp2$vcov
 	        h2 <- tmp2$coeff 			
 	        DFBETAS[i, ] <- (theta - h2)/sqrt(diag(vcovmat))
@@ -118,26 +109,23 @@ gCD <- function(data, model, na.rm = TRUE, digits = 5)
 	    DFBETAS <- round(DFBETAS,digits)
 	    ret <- list(dfbetas = DFBETAS, gCD = gCD)    
 	}
-	##OPENMX## if(class(model) == "MxRAMModel" || class(model) == "MxModel" ){		
-	##OPENMX## 	mxMod <- model
-	##OPENMX## 	fullmxData <- mxData(cov(data), type="cov",	numObs = nrow(data))
-	##OPENMX## 	fullMod <- mxRun(mxModel(mxMod, fullmxData), silent = TRUE)
-	##OPENMX## 	theta <- fullMod@output$estimate		
-	##OPENMX## 	gCD <- c()	
-	##OPENMX## 	DFBETAS <- matrix(0,nrow(data),length(theta))
-	##OPENMX## 	for(i in 1:nrow(data)){
-	##OPENMX## 		tmpmxData <- mxData(cov(data[-i,]), type="cov",	
-	##OPENMX## 			numObs = nrow(data)-1)
-	##OPENMX## 		tmpMod <- mxRun(mxModel(mxMod, tmpmxData), silent = TRUE)
-	##OPENMX## 		vcovmat <- solve(tmpMod@output$estimatedHessian)
-	##OPENMX## 		h2 <- tmpMod@output$estimate			
-	##OPENMX## 		DFBETAS[i, ] <- (theta - h2)/sqrt(diag(vcovmat))
-	##OPENMX## 		gCD[i] <- t(theta - h2) %*%  vcovmat %*% (theta - h2)  
-	##OPENMX## 	}	
-	##OPENMX## 	gCD <- round(gCD,digits)
-	##OPENMX## 	DFBETAS <- round(DFBETAS,digits)
-	##OPENMX## 	ret <- list(dfbetas = DFBETAS, gCD = gCD)
-	##OPENMX## }
+	if(class(model) == "character"){      
+        if(!require(lavaan)) require(lavaan)
+	    mod <- lavaan::sem(model, data=data, ...)
+	    theta <- coef(mod)
+	    gCD <- c()    
+	    DFBETAS <- matrix(0, N, length(theta))
+	    for(i in 1:nrow(data)){
+	        tmp <- lavaan::sem(model, data[-i, ], ...)
+	        vcovmat <- vcov(tmp)
+	        h2 <- coef(tmp)
+	        DFBETAS[i, ] <- (theta - h2)/sqrt(diag(vcovmat))
+	        gCD[i] <- t(theta - h2) %*%  vcovmat %*% (theta - h2)  
+	    }
+	    gCD <- round(gCD,digits)
+	    DFBETAS <- round(DFBETAS,digits)
+	    ret <- list(dfbetas = DFBETAS, gCD = gCD)    
+	}
 	class(ret) <- 'gCD'
 	ret	
 }
